<?php namespace CodeSpace\Database;

use PDO;
use PDOStatement;

class Database {

	/**
	 * PDO instance
	 */
    public PDO $db;

	/**
	 * Database connection
	 * @param string $dialect database dialect
	 * @param string $host database host
	 * @param string $db database name
	 * @param string $user database user
	 * @param string $password user password
	 */
	public function __construct(string $dialect, string $host, string $db, string $user, string $password) {
		$dsn = "$dialect:host=$host;dbname=$db";
		$this->db = new PDO($dsn, $user, $password);
		$this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    }

	/**
	 * Query list of items from DB
	 * @param string $sql SQL query
	 * @param array $array parameters
	 * @return array array of results
	 */
	public function query(string $sql, array $array = []): array {
        $statement = $this->db->prepare($sql);
        $statement->execute($array);
        return $statement->fetchAll(PDO::FETCH_ASSOC);
    }

	/**
	 * Run query on the DB
	 * @param string $sql SQL query
	 * @param array $array parameters
	 * @return bool true on success or false on failure
	 */
	public function run(string $sql, array $array = []): bool {
        $statement = $this->db->prepare($sql);
        return $statement->execute($array);
    }

	/**
	 * Run query on the DB and return raw statement
	 * @param string $sql SQL query
	 * @param array $array parameters
	 * @return PDOStatement|bool PDO statement od false on failure
	 */
	public function raw(string $sql, array $array = []): PDOStatement | bool {
        $statement = $this->db->prepare($sql);
		$statement->execute($array);
        return $statement;
    }

	/**
	 * Get first row of query from DB
	 * @param string $sql SQL query
	 * @param array $array parameters
	 * @return ?array first row or null
	 */
	public function row(string $sql, array $array = []): ?array {
        $resp = self::query($sql, $array);
        if(sizeOf($resp) == 0) return null;
        return $resp[0];
    }

}
